# This file is intentionally left empty to make the directory a Python package.
# The endpoints package contains the implementation of the different API endpoints.
# Each endpoint class is responsible for a specific category of endpoints.
# The MapHubClient class uses these endpoint classes to provide a unified API.
